#ifndef ENDOSCOPY_CPP_IMPLEMENTATION_ENDOSCOPY_ANNOTATION_XML_UTILITY_H
#define ENDOSCOPY_CPP_IMPLEMENTATION_ENDOSCOPY_ANNOTATION_XML_UTILITY_H

#pragma once
#include <iostream>
#include <fstream>
#include <sys/stat.h>
#include <string>
#include <sstream>
#include <vector>
#include <map>
#include <chrono>
#include <direct.h>
#include <pugixml.hpp>
#include <nlohmann/json.hpp>


#ifndef EXPORT_DLL
#define EXPORT_DLL

#define DLL_EXPORT __declspec(dllexport)
#define DLL_IMPORT __declspec(dllimport)

#endif

using namespace std;
using json = nlohmann::ordered_json;
using xml_node = pugi::xml_node;


DLL_EXPORT pugi::xml_document LoadXml(const string& path);
DLL_EXPORT map<string, string> ParseImageInformation(const pugi::xml_document& root);
DLL_EXPORT map<string, string> ParseUserInformation(const pugi::xml_document& root);
DLL_EXPORT vector<map<string, string>> ParseLabelList(const pugi::xml_document& root);
DLL_EXPORT vector<map<string, string>> ParseMask(const pugi::xml_document& root);
DLL_EXPORT vector<map<string, string>> ParseRectangleAnnotations(const pugi::xml_document& root);
DLL_EXPORT vector<map<string, string>> ParseLineAnnotations(const pugi::xml_document& root);
DLL_EXPORT vector<map<string, string>> ParsePolygonAnnotations(const pugi::xml_document& root);
DLL_EXPORT vector<map<string, string>> ParseCircleAnnotations(const pugi::xml_document& root);
DLL_EXPORT vector<map<string, string>> ParseFreehandContourAnnotations(const pugi::xml_document& root);
DLL_EXPORT vector<map<string, string>> ParseRulerAnnotations(const pugi::xml_document& root);
DLL_EXPORT vector<map<string, string>> ParseEllipseAnnotations(const pugi::xml_document& root);
DLL_EXPORT vector<map<string, string>> ParseTextAnnotations(const pugi::xml_document& root);
DLL_EXPORT vector<map<string, string>> ParseArrowTextAnnotations(const pugi::xml_document& root);
DLL_EXPORT vector<map<string, string>> ParsePolyLineAnnotations(const pugi::xml_document& root);
DLL_EXPORT vector<map<string, string>> ParseFreehandPolyLineAnnotations(const pugi::xml_document& root);
DLL_EXPORT vector<map<string, string>> ParseAngleAnnotations(const pugi::xml_document& root);
DLL_EXPORT vector<map<string, string>> ParsePointObjectAnnotations(const pugi::xml_document& root);
DLL_EXPORT vector<map<string, string>> ParseCobbAngleAnnotations(const pugi::xml_document& root);
DLL_EXPORT vector<map<string, string>> ParseDeviationAnnotations(const pugi::xml_document& root);
DLL_EXPORT map<string, vector<map<string, string>>> GetAllAnnotations(const pugi::xml_document& root);


DLL_EXPORT map<string, string> ParseImageInformation(const string& path);
DLL_EXPORT map<string, string> ParseUserInformation(const string& path);
DLL_EXPORT vector<map<string, string>> ParseLabelList(const string& path);
DLL_EXPORT vector<map<string, string>> ParseMask(const string& path);
DLL_EXPORT vector<map<string, string>> ParseRectangleAnnotations(const string& path);
DLL_EXPORT vector<map<string, string>> ParseLineAnnotations(const string& path);
DLL_EXPORT vector<map<string, string>> ParsePolygonAnnotations(const string& path);
DLL_EXPORT vector<map<string, string>> ParseCircleAnnotations(const string& path);
DLL_EXPORT vector<map<string, string>> ParseFreehandContourAnnotations(const string& path);
DLL_EXPORT vector<map<string, string>> ParseRulerAnnotations(const string& path);
DLL_EXPORT vector<map<string, string>> ParseEllipseAnnotations(const string& path);
DLL_EXPORT vector<map<string, string>> ParseTextAnnotations(const string& path);
DLL_EXPORT vector<map<string, string>> ParseArrowTextAnnotations(const string& path);
DLL_EXPORT vector<map<string, string>> ParsePolyLineAnnotations(const string& path);
DLL_EXPORT vector<map<string, string>> ParseFreehandPolyLineAnnotations(const string& path);
DLL_EXPORT vector<map<string, string>> ParseAngleAnnotations(const string& path);
DLL_EXPORT vector<map<string, string>> ParsePointObjectAnnotations(const string& path);
DLL_EXPORT vector<map<string, string>> ParseCobbAngleAnnotations(const string& path);
DLL_EXPORT vector<map<string, string>> ParseDeviationAnnotations(const string& path);
DLL_EXPORT map<string, vector<map<string, string>>> GetAllAnnotations(const string& path);

DLL_EXPORT void GenerateCOCOJson(const string& path, const string& output_path);

// Methods for Generating COCO Json
DLL_EXPORT string ParseLabelOnAnnotation(const pugi::xml_node& node);
DLL_EXPORT string Join(const vector<string>& vec, const string& delimeter);
DLL_EXPORT vector<double> CalculateBoundingBox(const vector<double>& points);
DLL_EXPORT string GetCurrentYear();
DLL_EXPORT string GetCurrentDateFormatted();
DLL_EXPORT std::vector<double> ParsePoints(const std::string& pointsStr);
DLL_EXPORT json GetMasks(const pugi::xml_document& root);
DLL_EXPORT json GetAnnotations(const pugi::xml_document& root);
DLL_EXPORT void GetRectangles(const pugi::xml_document& root, nlohmann::ordered_json& jsonAnnotations, int& annotationId);
DLL_EXPORT void GetCircles(const pugi::xml_document& root, nlohmann::ordered_json& jsonAnnotations, int& annotationId);
DLL_EXPORT void GetPolygons(const pugi::xml_document& root, nlohmann::ordered_json& jsonAnnotations, int& annotationId);

#endif //ENDOSCOPY_CPP_IMPLEMENTATION_ENDOSCOPY_ANNOTATION_XML_UTILITY_H
